##########################################################################
# Panel Functions
##########################################################################
##########################################################################
# One Step Estimator with Location shifts 
##########################################################################
rq.fit.cpanel <- function(X,y,s,w=c(.25,.5,.25),taus=(1:3)/4,lambda = 1){
	# Chamberlain (1982) and Abrevaya (2005) represenation of the alphas. Correlated case:
	# the matrix X is assumed to contain an intercept
	# the vector s is a strata indicator assumed (so far) to be a one-way layout
	# the vector t is a time indicator.
	# Simple case of 1 independent variable.
	require(SparseM)
	require(quantreg)
	K <- length(w)
	if(K != length(taus))
	stop("length of w and taus must match")
	X <- as.matrix(X)
	p <- ncol(X)
	n <- length(levels(as.factor(s)))
	N <- length(y)
	if(N != length(s) || N != nrow(X))
		 stop("dimensions of y,X,s must match")
	m <- N/n
	xx <- matrix(NA,n,m)
	for (q in 1:n){xx[q,] <- X[,2][s==q]}
	F <- matrix(rep(xx,each=m),m*n,m)
	pF <- ncol(F)
	Z <- as.matrix.csr(model.matrix(~as.factor(s)-1))
	Fidelity <- cbind(as(w,"matrix.diag.csr") %x% X, as.matrix.csr(w) %x% as.matrix.csr(F), as.matrix.csr(w) %x% Z)  
	Penalty <- cbind(as.matrix.csr(0,n,K*p+pF),lambda*as(n,"matrix.diag.csr"))
	D <- rbind(Fidelity,Penalty)
	y <- c(w %x% y,rep(0,n))
	a <- c((w*(1-taus)) %x% (t(X)%*%rep(1,N)), sum(w*(1-taus)) * (t(F) %*% rep(1,N)), sum(w*(1-taus)) * (t(Z) %*% rep(1,N)) + lambda/2 * rep(1,n))    
	rq.fit.sfn(D,y,rhs=a)
	}
##########################################################################
# One Step Estimator with Distributional shifts 
##########################################################################
rq.fit.cdpanel <- function(X,y,s,w=c(.25,.5,.25),taus=(1:3)/4, lambda = 1){
	# Chamberlain (1982) and Abrevaya (2005) represenation of the alphas. Correlated case:
	# the matrix X is assumed to contain an intercept, and the first covariates
	# need to be time-variant.
	# the vector s is a strata indicator assumed (so far) to be a one-way layout
	# the vector t is a time indicator.
	# Simple case of 1 independent variable.
	require(SparseM)
	require(quantreg)
	K <- length(w)
	if(K != length(taus))
	stop("length of w and taus must match")
	X <- as.matrix(X)
	p <- ncol(X)
	n <- length(levels(as.factor(s)))
	N <- length(y)
	if(N != length(s) || N != nrow(X))
		 stop("dimensions of y,X,s must match")
	m <- N/n
	xx <- matrix(NA,n,m)
	for (q in 1:n){xx[q,] <- X[,2][s==q]}
	F <- matrix(rep(xx,each=m),m*n,m)
	pF <- ncol(F)
	Z <- as.matrix.csr(model.matrix(~as.factor(s)-1))
	Fidelity <- cbind(as(w,"matrix.diag.csr") %x% X, as(w,"matrix.diag.csr") %x% as.matrix.csr(F), as.matrix.csr(w) %x% Z)  
	Penalty <- cbind(as.matrix.csr(0,n,K*(p+pF)),lambda*as(n,"matrix.diag.csr"))
	D <- rbind(Fidelity,Penalty)
	y <- c(w %x% y,rep(0,n))
	a <- c((w*(1-taus)) %x% (t(X)%*%rep(1,N)), (w*(1-taus)) %x% (t(F) %*% rep(1,N)), sum(w*(1-taus)) * (t(Z) %*% rep(1,N)) + lambda/2 * rep(1,n))
	rq.fit.sfn(D,y,rhs=a,control=list(tmpmax=floor(1e5+exp(-12.1)*(as.matrix.csr(X)@ia[p+1]-1)^2.35)))
	}
##########################################################################
# Two Steps Estimator 
##########################################################################
rq.fit.sppanel <- function(X,y,s,k=KK,w=c(.25,.5,.25),taus=(1:3)/4,lambda = 1,degree = 3){
	require(SparseM)
	require(quantreg)
	K <- length(w)
	if(K != length(taus))
	stop("length of w and taus must match")
	X <- as.matrix(X)
	p <- ncol(X)
	n <- length(levels(as.factor(s)))
	N <- length(y)
	if(N != length(s) || N != nrow(X))
	 stop("dimensions of y,X,s must match")
	m <- N/n
	xx <- array(NA,c(n,m,k))
	for (j in 1:k){for (q in 1:n){xx[q,,j] <- X[,(j+1)][s==q]}}
	F <- matrix(rep(xx,each=m),m*n,m*k)
	pF <- ncol(F)
	Fs <- NULL
	for (jj in 1:pF){Fs <- cbind(Fs,bs(F[,jj],degree=degree))}
	pFs <- ncol(Fs)
	Z <- as.matrix.csr(model.matrix(~as.factor(s)-1))
	Fidelity <- cbind(as(w,"matrix.diag.csr") %x% X, as.matrix.csr(w) %x% as.matrix.csr(Fs), as.matrix.csr(w) %x% Z)  
	Penalty <- cbind(as.matrix.csr(0,n,K*p+pFs),lambda*as(n,"matrix.diag.csr"))
	D <- rbind(Fidelity,Penalty)
	y <- c(w %x% y,rep(0,n))
	a <- c((w*(1-taus)) %x% (t(X)%*%rep(1,N)), sum(w*(1-taus)) * (t(Fs) %*% rep(1,N)), sum(w*(1-taus)) * (t(Z) %*% rep(1,N)) + lambda/2 * rep(1,n))    
	rq.fit.sfn(D,y,rhs=a,control=list(tmpmax=floor(1e5+exp(-12.1)*(as.matrix.csr(Fidelity)@ia[(p+pF)]-1)^2.35)))
}
##########################################################################
# CANAY's ESTIMATOR: 
# Note: For questions on this code, contact Professor Ivan Canay at NU
##########################################################################
pqr.estimator <-function(dataframe,eqformula,tau){
	#---
	# Author: Ivan Canay | Northwestern University | January 2001
	# Copyright (c) 2011 Northwestern University. All rights reserved.
	#---
	# INPUTS: 	- eqformula: Y ~ X1 + X2 + etc. (intercept is then added)
	#			- tau: the quantile - number in (0,1)
	#			- dataframe: Must have the following structure
	#	 firm year  Yit       X1it      X2it  etc
	#1    1    1  4.0651708 0.7475937
	#2    1    2 -0.7227345 0.2987556
	#3    1    3  2.4296410 0.5015247
	#4    1    4  0.3724746 0.4901809
	#5    1    5 -0.6716404 0.2786336
	#6    1    6  1.6568098 0.1311757
	#.    .    .  ......... .........
	#  IMPORTANT = "firm" and "year" must have that name exactly
	#--
	# RETURNS: This function a structure with the following element. 
	#       - theta    = estimated theta(\tau): K x 1 vector.
	#	   - prelim   = preliminary estimator theta_{mu}: K x 1.
	#	  - fe = nT x 1 vector of estimated fixedeffects
	#       - df       = dataframe(firm, year, hatY_{it}, X_{it})
	#        - tau      = quantile
	#  Standard errors are computed using a different function. 
	#  See `pqr.se' 
	#--
	
	data    <- pdata.frame(dataframe, index = c("firm", "year"), drop.index = TRUE, row.names = TRUE)
	within 	<- plm(eqformula,data=data,model="within")
	prelim 	<- cbind(coef(within))
	hatalpha.i  <- cbind(fixef(within))
	intercept   <- mean(hatalpha.i)
	unos.T      <-rep(1,sum(dataframe[,1]==dataframe[1,1]))
	hatalpha    <- kronecker(hatalpha.i-intercept,unos.T) 
		
	dataframe[,3] <- dataframe[,3]-hatalpha
	hattheta   <- coef(rq(eqformula,tau,data=dataframe))
	list(theta=cbind(hattheta),prelim=rbind(intercept,prelim),fe=hatalpha,df=dataframe,tau=tau)
	}
##########################################################################
# MISC codes 
##########################################################################
lambda.star.iid.NN <- function(weight=0.5,tau=0.5,siga=1,sigu=1)
{
	f = sigu*dnorm(qnorm(tau))
	g = siga*dnorm(0)
	Omega <- weight*tau*(1-tau)*weight
	de <- 8*Omega*g
	nu <- (weight*f)
	star.lambda <- de/nu
	list(f=f,g=g,star.lambda=star.lambda)
	}
##########################################################################
mlm.fit <- function (x,y,s,theta)
{
     xm <-tapply(x,s,mean)[s]
     ym <-tapply(y,s,mean)[s]
     xt <- matrix(x - (1 - theta)*xm)
     yt <- matrix(y - (1 - theta)*ym)
     lm(yt ~ xt)
}