##########################################################################
##########################################################################
#
#  This file reproduces the results in Table 4.1 
#  of "PENALIZED QUANTILE REGRESSION with SEMIPARAMETRIC CORRELATED 
#  EFFECTS: AN APPLICATION with HETEROGENEOUS PREFERENCES" 
#  by M. Harding and C. Lamarche. 
#  October, 2015
#
##########################################################################
##########################################################################
#
# CREATED     :   September 4, 2013 at 17:49:21 EDT
# MODIFIED    :   October 12, 2015 at 08:56:31 EDT 
date()
# AUTHORS     :   Matthew Harding and Carlos Lamarche
# AFFILIATIONS:   Duke University and University of Kentucky
# EMAIL       :   matthew.harding@duke.edu, clamarche@uky.edu, 
# SESSION INFORMATION:
sessionInfo()
#
##########################################################################
##########################################################################
##########################################################################
# Libraries and Functions
##########################################################################
library(quantreg)
library(splines)
library(SparseM)
library(plm)
source("pen.jae.R")
#########################################################################
# Data
##########################################################################
n <- c(100,100,500,500)
m <- c(2,12,2,12)
R <- 400
gamma = 2
sigs <- c(sqrt(1),sqrt(1))
lbda.qr <- c(100)
F00 <- array(0,c(11,R,length(n)))
G00 <- array(0,c(11,R,length(n)))
#
for (ii in 1:length(n))
	{
	s <- rep(1:n[ii],each=m[ii])
	set.seed(14)
	xit  <- matrix(rbeta(n[ii]*m[ii],1,1),n[ii],m[ii])
	Xit  <- as.matrix(c(t(xit))) 
	x <- Xit
	X <- cbind(1,x)
	barX.i <- tapply(Xit,s,mean) 
	xx <- matrix(NA,n[ii],m[ii])
	for (q in 1:n[ii]){xx[q,] <- X[,2][s==q]}
	F <- matrix(rep(xx,each=m[ii]),m[ii]*n[ii],m[ii])
	t <- rep(1:m[ii],n[ii])
	ZF <- as.matrix.csr(model.matrix(t~as.factor(t)-1))[,-1]
	lambdasF00 <- c(lambda.star.iid.NN(siga=sigs[1],sigu=sigs[2])$star.lambda, lbda.qr[1]) 
	lambdasG00 <- c(lambda.star.iid.NN(siga=sigs[1],sigu=sigs[2])$star.lambda, lbda.qr[1]) 
	nlambdas00 <- length(lambdasF00)
	thetas00 <- c(sqrt(sigs[2]^2/(sigs[2]^2 + m[ii]*sigs[1]^2)),1)
	nthetas00 <- length(thetas00)
	#
	for(i in 1:R){
		E  <- matrix(rnorm(n[ii]*m[ii])+2,n[ii],m[ii])
		Eit  <- as.matrix(c(t(E)))
		V <- c(rnorm(n[ii]))
		A <- cbind(gamma*(m[ii]*barX.i + V))
		A <- A - mean(A)     
		alpha.i <- as.matrix(kronecker(A,rep(1,m[ii])))
		yF00 <- Eit-1 + Xit*Eit + alpha.i
		#
		for(j in 1:1){
			lmbd <- lambda.star.iid.NN(siga=sigs[1],sigu=sigs[2])$star.lambda
			F00[j,i,ii] <- rq.fit.cpanel(X,yF00,s,lambda=lmbd)$coef[4]
			G00[j,i,ii] <- rq.fit.cpanel(X,yF00,s,lambda=lmbd)$coef[2]
			}
		for(j in 2:2){
			lmbd <- lambda.star.iid.NN(siga=sigs[1],sigu=sigs[2])$star.lambda
			F00[j,i,ii] <- rq.fit.cdpanel(X,yF00,s,lambda=lmbd)$coef[4]
			G00[j,i,ii] <- rq.fit.cdpanel(X,yF00,s,lambda=lmbd)$coef[2]
			}
		for(j in 3:3){
			lmbd <- lambda.star.iid.NN(siga=sigs[1],sigu=sigs[2])$star.lambda
			F00[j,i,ii] <- rq.fit.sppanel(X,yF00,s,k=1,lambda=lmbd)$coef[4]
			G00[j,i,ii] <- rq.fit.sppanel(X,yF00,s,k=1,lambda=lmbd)$coef[2]
			}
		for(j in 4:4){
			firm <- rep(1:n[ii],each=m[ii])
			year <- rep(1:m[ii],n[ii])
			DF00 <-data.frame(firm,year,yF00,x)
			F00[j,i,ii] <- pqr.estimator(DF00,yF00 ~ x,tau=0.5)$theta[2]
			G00[j,i,ii] <- pqr.estimator(DF00,yF00 ~ x,tau=0.25)$theta[2]
			}
		for(j in 5:5){
			F00[j,i,ii] <- rq(yF00 ~ X + as.matrix(ZF) + F - 1,tau=0.5)$coef[2]
			G00[j,i,ii] <- rq(yF00 ~ X + as.matrix(ZF) + F - 1,tau=0.25)$coef[2]
			}
		for(j in 1:nlambdas00){
			F00[5+j,i,ii] <- rq.fit.panel(X,yF00,s,lambda=lambdasF00[j])$coef[4]
			G00[5+j,i,ii] <- rq.fit.panel(X,yF00,s,lambda=lambdasG00[j])$coef[2]
			}
		for(j in 1:length(thetas00)){
			F00[7+j,i,ii] <- mlm.fit(c(x),c(yF00),s,theta=thetas00[j])$coef[2]
			G00[7+j,i,ii] <- mlm.fit(c(x),c(yF00),s,theta=thetas00[j])$coef[2]
			}
	}
}
#
# Functions to obtain bias and rmse
#
	bias <- function(x,tau){(mean(x) - (qnorm(tau)+2))/(qnorm(tau)+2)}
	rmse <- function(x,tau){((mean(x) - qnorm(tau)-2)^2 + var(x))^.5}
#
# Generate a Table
#
B <- array(NA,c(8,9))
colnames(B) <- c("OLS","GLS", "QR", "PFE", "CQR", "2SQR", "SQR", "PQRd","PQR")
rownames(B) <- c("Bias", "RMSE","Bias", "RMSE","Bias", "RMSE","Bias", "RMSE")
for (i in 1:length(colnames(B))) {B[1,i] <- bias(F00[length(colnames(B))+1-i,,1],tau=0.5)}
for (i in 1:length(colnames(B))) {B[2,i] <- rmse(F00[length(colnames(B))+1-i,,1],tau=0.5)}
for (i in 1:length(colnames(B))) {B[3,i] <- bias(F00[length(colnames(B))+1-i,,2],tau=0.5)}
for (i in 1:length(colnames(B))) {B[4,i] <- rmse(F00[length(colnames(B))+1-i,,2],tau=0.5)}
for (i in 1:length(colnames(B))) {B[5,i] <- bias(F00[length(colnames(B))+1-i,,3],tau=0.5)}
for (i in 1:length(colnames(B))) {B[6,i] <- rmse(F00[length(colnames(B))+1-i,,3],tau=0.5)}
for (i in 1:length(colnames(B))) {B[7,i] <- bias(F00[length(colnames(B))+1-i,,4],tau=0.5)}
for (i in 1:length(colnames(B))) {B[8,i] <- rmse(F00[length(colnames(B))+1-i,,4],tau=0.5)}
#
round(B,3)
#
Bt <- array(NA,c(8,9))
colnames(Bt) <- c("OLS","GLS", "QR", "PFE", "CQR", "2SQR", "SQR", "PQRd","PQR")
rownames(Bt) <- c("Bias", "RMSE","Bias", "RMSE","Bias", "RMSE","Bias", "RMSE")
for (i in 1:length(colnames(Bt))) {Bt[1,i] <- bias(G00[length(colnames(Bt))+1-i,,1],tau=0.25)}
for (i in 1:length(colnames(Bt))) {Bt[2,i] <- rmse(G00[length(colnames(Bt))+1-i,,1],tau=0.25)}
for (i in 1:length(colnames(Bt))) {Bt[3,i] <- bias(G00[length(colnames(Bt))+1-i,,2],tau=0.25)}
for (i in 1:length(colnames(Bt))) {Bt[4,i] <- rmse(G00[length(colnames(Bt))+1-i,,2],tau=0.25)}
for (i in 1:length(colnames(Bt))) {Bt[5,i] <- bias(G00[length(colnames(Bt))+1-i,,3],tau=0.25)}
for (i in 1:length(colnames(Bt))) {Bt[6,i] <- rmse(G00[length(colnames(Bt))+1-i,,3],tau=0.25)}
for (i in 1:length(colnames(Bt))) {Bt[7,i] <- bias(G00[length(colnames(Bt))+1-i,,4],tau=0.25)}
for (i in 1:length(colnames(Bt))) {Bt[8,i] <- rmse(G00[length(colnames(Bt))+1-i,,4],tau=0.25)}
#
round(Bt,3)

